﻿using Infragistics.Olap;
using Infragistics.Olap.Data;
using Infragistics.Olap.FlatData;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Collections.Specialized;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

namespace XamPivotGridValidationTooltipDemo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();

            //SalesDataSample data = new SalesDataSample();

            //FlatDataSource fds = new FlatDataSource() {  ItemsSource = data};
            

            //grid.DataSource = fds;            
            //selector.DataSource = fds;

            grid.DataSource.Measures.CollectionChanged += Measures_CollectionChanged;
            grid.CellEdited += Grid_CellEdited;
            grid.CellEditing += Grid_CellEditing;
        }

        private void Grid_CellEditing(object sender, Infragistics.Controls.Grids.PivotCellEditingEventArgs e)
        {
            if (e.Measure.Name == "NumberOfUnits")
            {
                EventHandler<ValidationErrorEventArgs> evh = OnValueValidationError;
                Validation.AddErrorHandler(this, evh);
                ValueValidation obj = new ValueValidation { Value = e.EditedValue };
                Binding b = new Binding("Value");
                b.Source = obj;
                b.ValidationRules.Add(new ValueValidation { Min = 0, Max = 10000 });
                b.ValidatesOnDataErrors = b.ValidatesOnExceptions = b.NotifyOnValidationError = true;

                e.Editor.SetBinding(TextBox.TextProperty, b);
                e.Editor.GetBindingExpression(TextBox.TextProperty).UpdateSource();

                // handle validation errors here or in OnValueValidationError event handler
                if (Validation.GetHasError(e.Editor))
                {
                    e.Cancel = true;
                    ReadOnlyObservableCollection<ValidationError> errors = Validation.GetErrors(e.Editor);
                    foreach (var error in errors)
                    {
                        // notify the user about each validation error
                        string message = "Invalid value detected: " + e.Editor.Text;
                        message += Environment.NewLine + error.ErrorContent;
                        //MessageBox.Show(message, "Validation Error", MessageBoxButton.OK);
                    }
                }

                Validation.RemoveErrorHandler(this, evh);
            }
        }

        private void OnValueValidationError(object sender, ValidationErrorEventArgs e)
        {
            if (e.Action == ValidationErrorEventAction.Added)
            {
                // notify the user about validation error
                string message = e.Error.ErrorContent.ToString();                
                //MessageBox.Show(message, "Validation Error", MessageBoxButton.OK);
            }
        }

        private void Grid_CellEdited(object sender, Infragistics.Controls.Grids.PivotCellEditedEventArgs e)
        {
            double newValue = Double.Parse(e.EditedValue.ToString());
            List<int> indexes = ((FlatDataSource)this.grid.DataSource)
                .GetCellItemsIndexes(e.Cell.Data as ICell);
            if (e.Measure.Caption == "Units sold")
            {
                Sale sale;
                foreach (int index in indexes)
                {
                    sale = ((FlatDataSource)this.grid.DataSource).GetRecord(index) as Sale;
                    if (sale != null) sale.NumberOfUnits = 0;
                }
                sale = ((FlatDataSource)this.grid.DataSource).GetRecord(indexes[0]) as Sale;
                if (sale != null) sale.NumberOfUnits = (int)newValue;
            }
            else
            {
                foreach (int index in indexes)
                {
                    Sale sale = ((FlatDataSource)this.grid.DataSource).GetRecord(index) as Sale;
                    if (sale != null) sale.UnitPrice = newValue;
                }
            }
            this.grid.DataSource.RefreshGrid();
        }

        private void Measures_CollectionChanged(object sender, System.Collections.Specialized.NotifyCollectionChangedEventArgs e)
        {
            switch (e.Action)
            {
                case NotifyCollectionChangedAction.Add:
                    foreach (IMeasureViewModel mvm in e.NewItems)
                    {
                        if (mvm.Caption != "Amount of sale")
                            this.grid.EditSettings.EditableMeasures.Add(mvm.Measure);
                        if (mvm.Caption == "Unit price")
                        {
                            this.grid.DataSource.SetMeasureAggregator(
                                (IMeasureViewModel)mvm,
                                AggregationHelper.GetDefaultAverageAggregator(typeof(double))
                            );
                        }
                    }
                    break;
                case NotifyCollectionChangedAction.Remove:
                    foreach (IMeasureViewModel mvm in e.OldItems)
                    {
                        this.grid.EditSettings.EditableMeasures.Remove(mvm.Measure);
                    }
                    break;
            }
        }
    }
}
